#ifndef md5_INCLUDED
#define md5_INCLUDED

#define MD5_BLOCK_LENGTH  64
#define MD5_DIGEST_LENGTH 16

/** The state of the MD5 Algorithm. */
struct MD5_CTX {
  /** message length in bits, lsw first */
  u_int32_t count[2];
  /** digest buffer */
  u_int32_t abcd[4];
  /** accumulate block */
  unsigned char  buf[MD5_BLOCK_LENGTH];
};

/** Initialize the state object pms. */
void MD5Init(MD5_CTX *pms);

/** Append data to the message. */
void MD5Update(MD5_CTX *pms, const void *data, int nbytes);

/** Finish the message and return the digest. */
void MD5Final(unsigned char digest[MD5_DIGEST_LENGTH], MD5_CTX *pms);

/**
   The MD5End() function is a wrapper for MD5Final() which converts
   the return value to a 33-character (including the terminating '\0')
   ASCII string which represents the 128 bits in hexadecimal. The buf
   argument can be a null pointer, in which case the returned string
   is allocated with malloc(3) and subsequently must be explicitly
   deallocated using free(3) after use.  If the buf argument is
   non-null it must point to at least 33 characters of buffer space.

   @param pms pointer to MD5 context
   @param buf pointer to digest text string. Can be NULL.

   @return digest as text. If buf is NULL this is a malloc()ed buffer, otherwise buf is returned.
 */
char* MD5End(MD5_CTX *pms, char *buf);

/**
   The MD5Data() function calculates the digest of a chunk of data in
   memory and returns the digest as text. The buf argument can be a
   null pointer, in which case the returned string is allocated with
   malloc(3) and subsequently must be explicitly deallocated using
   free(3) after use. If the buf argument is non-null it must point to
   at least 33 characters of buffer space.

   @param data pointer to data buffer
   @param len length of data
   @param buf pointer to digest text string. Can be NULL.

   @return digest as text. If buf is NULL this is a malloc()ed buffer, otherwise buf is returned.
 */
char* MD5Data(const void *data, int len, char *buf);

/**
   convert data to hexadecimal notation. If buf is NULL a string is
   malloc()ed and returned which must be free()ed by the caller.

   @param data pointer to data
   @param len length of data
   @param buf pointer to string of at least len*2+1 bytes or NULL

   @return buf or malloc()ed memory, NULL upon error
*/
char *binary2hex(unsigned char *data, unsigned len, char *buf);

#ifdef __cplusplus
#include <string>

/** convert MD5 digest of s and return as hex-string */
std::string MD5Data(const std::string& s);

#endif /* __cplusplus */

#endif /* md5_INCLUDED */
